/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 15/10/22.
//

#ifndef FGGL_INTEGRATIONS_LUA_SCRIPT_LUA_ENGINE_HPP
#define FGGL_INTEGRATIONS_LUA_SCRIPT_LUA_ENGINE_HPP

#include "fggl/script/engine.hpp"
#include "fggl/data/storage.hpp"

extern "C" {
	#include <lua.h>
	#include <lauxlib.h>
	#include <lualib.h>
};

namespace fggl::script::lua {

	class LuaScriptEngine : public ScriptEngine {
		public:
			LuaScriptEngine(data::Storage* storage);
			virtual ~LuaScriptEngine();

			void onActivate() override;
			void onDeactivate() override;
			void onUpdate() override;

			void onEvent(const std::string& name) override;

			// running scripts
			bool run(const char* script) override;
			bool load(const char* filename) override;

			// variables
			void setGlobal(const char* name, void* ptr) override;

		private:
			data::Storage* m_storage;
			lua_State* m_state;
			void release();
	};

	class LuaScriptProvider : public ScriptProvider {
		public:
			LuaScriptProvider(data::Storage* storage);

			virtual ~LuaScriptProvider() = default;
			LuaScriptEngine* create() override;

		private:
			data::Storage* m_storage;
	};

}

#endif //FGGL_INTEGRATIONS_LUA_SCRIPT_LUA_ENGINE_HPP
